
% Any publication resulting from the use of this m-file shall acknowledge
% it by citing the following paper:

%S. P. Torres and C. A. Castro, Expansion planning for smart transmission
%grids using ac model and shunt compensation, IET Generation,
%Transmission & Distribution, vol. 8, no. 5, pp. 966-975, 2014.


%% IMPLEMENTATION OF UNIFIED PSO (UPSO) %%
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 function [xopt,fxopt,fevalcount, FOPT]= PSO(MaxIt,Max_FES,SS,dim,Xmin,Xmax,GM,nexp,fhd, sistema,sistema_prueba,varargin)
 
% Initialization of parameters needed for PSO
 clc;
 rand('state',sum(100*clock)); % Different random generator seeds per execution
 
 Acc = 1e-3; % Desired accuracy

 RunStats = [ ]; % Matrix of average statistics
 
 chi = 0.729; % Constriction coefficient
 c1 = 2.05; % Cognitive parameter
 c2 = 2.05; % Social parameter
 NR = 1; % Vecinity ratio(NR < SS should hold)

 vmax = 0.5*(Xmax-Xmin); % Maximum veolcity
 
 xopt=zeros(dim, nexp);
 FOPT=zeros(MaxIt, nexp);
  mpc=loadcase(sistema);


 %===================================
 % Iterative loop
 %===================================
 for T = 1:nexp  %Number of Monte-Carlo simulations

 %Calculation of demand and generation values
 %------------------------------------------------------
 %%Please this does not change%%   
 switch sistema_prueba
        
    case {'garver'}
     [pde1,pde2,pde3,pde4,pde5,pge] = uncertainty_garver;%%random values (generated from the distribution of probability normal and Weibull           
     sistema=garver(pde1,pde2,pde3,pde4,pde5,pge);  

    case {'garver_WOrdptch'}
     [pde1,pde2,pde3,pde4,pde5,pge] =uncertainty_garver;%random values (generated from the distribution of probability normal and Weibull          
     sistema=garver_WOrdptch(pde1,pde2,pde3,pde4,pde5,pge);            
            
    case {'case24IEEE_AC'}
     [pde1,pde2,pde3,pde4,pde5,pde6,pde7,pde8,pde9,pde10,pde11,...
      pde12,pde13,pde14,pde15,pde16,pde17,pge1,pge2,pge3] =uncertainty_24IEEE;%random values (generated from the distribution of probability normal and Weibull
     sistema=case24IEEE_AC(pde1,pde2,pde3,pde4,pde5,pde6...
       ,pde7,pde8,pde9,pde10,pde11,pde12,pde13,pde14,pde15,pde16,pde17,pge1,pge2,pge3); 
   
     case {'case24IEEE_AC_WOrdptch'}
     [pde1,pde2,pde3,pde4,pde5,pde6,pde7,pde8,pde9,pde10,pde11,...
      pde12,pde13,pde14,pde15,pde16,pde17,pge1,pge2,pge3] =uncertainty_24IEEE;%random values (generated from the distribution of probability normal and Weibull
     sistema=case24IEEE_AC_WOrdptch(pde1,pde2,pde3,pde4,pde5,pde6...
       ,pde7,pde8,pde9,pde10,pde11,pde12,pde13,pde14,pde15,pde16,pde17,pge1,pge2,pge3); 
   
 end        
 %--------------------------------------------------------------------------------------------    
 % INITIALIZE PARAMETERS
     success = 0; % Exit flag
     iter = 1; % Iteration counter
     fevalcount = 0; % Counter of objective function evaluations
     STOP = 0; % Stop flag
     opcion=3; % Creation of initial solutions (1 by default)
 
 % SWARM, VELOCITUES & INICIALIZATION OF BEST POSITION
 
    %Call psocreacioninicial function to get initial solutions
    swarm=psocreacioninicial(Xmin, Xmax, dim, SS, sistema, fhd, opcion);
    
     vel = (2*rand(dim, SS)).*(Xmax/2)-(Xmax/2); %Velocity of initial population
     bestpos = swarm; % Initial best position

% Thi part of the code calculates the costs, considering static penalty
% which can be used by default (uncomment relevant commands and comment
% the commands below the uncommendted commands)

     
   for i=1:SS
      [costo_lineas(i), penalizacion(i)]=feval(fhd, swarm(:,i), sistema);
      fswarm(i)=costo_lineas(i)+penalizacion(i);
      fevalcount=fevalcount+1;  
    end


     
      fbestpos=fswarm; % Inicialization costs of best position
     
     % UPDATE BEST PARTICLE INDEXES
     [fxopt, g_over] = min(fbestpos);
     xopt(:,T) = bestpos(:,g_over);
     
     FOPT(iter,T)=fxopt;
  
   

     
     %======================
     % LOOP OF EVOLUTION OF THE SWARM
     %======================
     while (STOP == 0)
         
         % UPDATE ITERATION COUNTER
         iter = iter+1;
         
         % CREATE DYNAMIC UNIFICATION FACTOR
         
%          u=1/(1+exp(-lambda*(iter-MaxIt/20)));
%            u=iter/MaxIt;
                 
         % UPDATE VELOCITIES (MOVEMENT EQUATIONS OF THE SWARM)
         for i=1:SS
             
             % Global component PSO 
             R1 = rand(dim,1);
             R2 = rand(dim,1);
         
             G = chi .* (vel(:,i) + c1.*R1.*(bestpos(:,i)-swarm(:,i)) + c2.*R2.*(bestpos(:,g_over)-swarm(:,i)));



             % Update velocity of standard UPSO 
             vel(:,i) = G; 
         end
         % VELOCITY CONSTRAINT
         for i=1:dim
             for j=1:SS
                 if vel(i,j)>vmax(i,j)
                     vel(i,j)=vmax(i,j);
                 elseif vel(i, j)< -vmax(i, j);
                     vel(i,j)=-vmax(i,j);
                 end
             end
         end
         
         % UPDATE SWARM
         swarm = round(swarm + vel);
         
         % SWARM CONSTRAINT
        
        for i=1:SS
             for j=1:dim
                 if swarm(j,i)>Xmax(j,i)
                     swarm(j,i) = Xmax(j,i);
                 end
                 if swarm(j,i)<Xmin(j,i)
                     swarm(j,i) = Xmin(j,i);
                 end
             
             end
         end

     parfor i=1:SS
      [costo_lineas(i), penalizacion(i)]=feval(fhd, swarm(:,i), sistema);
      fswarm(i)=costo_lineas(i)+penalizacion(i);
      fevalcount=fevalcount+1;  
    end

         %UPDATE BEST POSITIONS
         bestpos(:,fswarm<fbestpos) = swarm(:,fswarm<fbestpos);
         fbestpos(fswarm<fbestpos) = fswarm(fswarm<fbestpos);
        
         
         % UPDATE SWARM INDEXES(GENERAL)
         [fxopt, g_over] = min(fbestpos);
         xopt(:,T) = bestpos(:,g_over); 
         

         
         % STORE AND PLOT OPTIMAL VALUES FOR EACH ITERATION
         
         FOPT(iter, T)=fxopt;
         
         figure(1);
         plot(FOPT(:, T)');
         xlabel('Iter');
         ylabel('Fitness (x 1000 US$)');
         hold on; 
                
         % CHECK STOP CRITERION
         
        if ((fevalcount >= Max_FES)||(fxopt <= GM+Acc))
         STOP = 1;
             if (fxopt <= GM+Acc)
             success = 1;
             end
        end
         
         
         if ((iter >= MaxIt)||(fxopt <= GM+Acc))
         STOP = 1;
             if (fxopt <= GM+Acc)
             success = 1;
             end
         end
        
         
       
 end % End of loop of swarm evolution

     
 end % END OF LOOP OF NUMBER OF MONTE-CARLON SIMULATIONS
 toc


xopt=round(xopt)
FOPT






 