%%% Script for initial parameters needed to solve the transmission
%%% expansion planning (TEP) problem by using AC or DC power flow calculation
%%% and also for running TEP (master script0


% Any publication resulting from the use of this m-file shall acknowledge
% it by citing the following paper:

%S. P. Torres and C. A. Castro, Expansion planning for smart transmission
%grids using ac model and shunt compensation, IET Generation,
%Transmission & Distribution, vol. 8, no. 5, pp. 966-975, 2014.
clear
for i=1:1
    
    % i this parameter helps to define the population size

    
    %% Test Systems
          sistema_prueba='garver'; %With Dispatchable generation, with unlimited shunt compensation

         %sistema_prueba='garver_WOrdptch'; %With Non-dispatchable generation, with unlimited shunt compensation

         %sistema_prueba='case24IEEE_AC';%With Dispatchable generation, with unlimited shunt compensation
 
         %sistema_prueba='case24IEEE_AC_WOrdptch'; %With Non-dispatchable generation, with unlimited shunt compensation

     
   %% Simulation parameters
    
    Particle_Number=i*50; %Number of individuals 
    Max_Gen=60;  %Maximum number of iterations or generations (stop criterion)
    Max_FES=5000; %Maximum number of objective function evaluations (alternative criterion)
    Xmax=10; % Maximum number of transmission lines that can be added in each right-of-way 
    
    nexp=500;  % Number of Monte-Carlo simulations
    GM=0; %Theoretical optimum value (used zero for all of the case)
    
 switch sistema_prueba
        
    case {'garver'}
     [pde1,pde2,pde3,pde4,pde5,pge]=uncertainty_garver;%random values (generated from the distribution of probability normal and Weibull           
     sistema=garver(pde1,pde2,pde3,pde4,pde5,pge);  
     
    case {'garver_WOrdptch'}
     [pde1,pde2,pde3,pde4,pde5,pge] =uncertainty_garver;%%random values (generated from the distribution of probability normal and Weibull            
     sistema=garver_WOrdptch(pde1,pde2,pde3,pde4,pde5,pge);            
            
    case {'case24IEEE_AC'}
     [pde1,pde2,pde3,pde4,pde5,pde6,pde7,pde8,pde9,pde10,pde11,...
      pde12,pde13,pde14,pde15,pde16,pde17,pge1,pge2,pge3]=uncertainty_24IEEE;%random values (generated from the distribution of probability normal and Weibull 
     sistema=case24IEEE_AC(pde1,pde2,pde3,pde4,pde5,pde6...
       ,pde7,pde8,pde9,pde10,pde11,pde12,pde13,pde14,pde15,pde16,pde17,pge1,pge2,pge3); 

     case {'case24IEEE_AC_WOrdptch'}
     [pde1,pde2,pde3,pde4,pde5,pde6,pde7,pde8,pde9,pde10,pde11,...
      pde12,pde13,pde14,pde15,pde16,pde17,pge1,pge2,pge3]=uncertainty_24IEEE;%random values (generated from the distribution of probability normal and Weibull 
     sistema=case24IEEE_AC_WOrdptch(pde1,pde2,pde3,pde4,pde5,pde6...
       ,pde7,pde8,pde9,pde10,pde11,pde12,pde13,pde14,pde15,pde16,pde17,pge1,pge2,pge3); 
 end
               
    mpc=loadcase(sistema);  % Load system data (defined in separate m-file)
    LineasIniciales=mpc.costo_lineas(:,1); 
    Costos=mpc.costo_lineas(:,2); % Store line cost data
    dim=numel(LineasIniciales);  %Problem dimension (defined by number of right-of-way considered for expansion) 

    
    Xmax = repmat(Xmax, dim, Particle_Number); 
    
    Xmin = LineasIniciales'; 
    Xmin=repmat(Xmin, Particle_Number,1)'; % Initial topology

    fhd=@evaluacion_particulas;

    
    tic % Start measuring cpu time
    
    % Call optimization function(defined by user/algorithm developer)

  [xopt,fxopt,fevalcount, FOPT]=PSO(Max_Gen,Max_FES,Particle_Number,dim,Xmin,Xmax, GM, nexp, fhd, sistema,sistema_prueba);  %%% Write here the name file of your algorithm (instead of 'PSO')

  t(i)=toc; % Stop measuring cpu time
  
%Calculation of results
%_______________________________________________________
FOPT_min=FOPT(end,:);
[valores_unicos8, ind_unicos8] = unique(FOPT_min');
gh=size(valores_unicos8);
valores_unicos_T=valores_unicos8';

for ii=1:gh(1)
incer_valo(ii)=numel(find(FOPT_min==valores_unicos_T(ii)));
probabilidad_aparecer_w(ii)=(100*incer_valo(ii)/nexp);
probabilidad_aparecer(ii)=valores_unicos_T(ii)*incer_valo(ii)/nexp;
end

figure(2);
Histograma=bar(probabilidad_aparecer_w,'LineWidth',2);
set(gca,'XTickLabel',valores_unicos_T);
set(gca,'XScale');
xlabel('Lines cost');
ylabel('Probability (%)');
filename1=strcat(sistema_prueba, '_', 'Histogram');  % Create a mat file
saveas(gca, fullfile( filename1), 'jpeg');
Valor_esperado=sum(probabilidad_aparecer);   
[incertudumbre_rep,po_incertidum]=max(incer_valo);

kss = find(FOPT_min==valores_unicos_T(po_incertidum));
xopt_best1=zeros(size(xopt,1),size(kss,2));
for ii=1:size(kss,2)
   xopt_best1(:,ii)= xopt(:,kss(ii));
end
[xopt_best, ind_unicos1] = unique(xopt_best1','rows');

soluxion_optima=xopt_best'    
%_______________________________________________________    
    Particle_name=num2str(Particle_Number);  % Transform particle number to string
    filename=strcat(sistema_prueba, '_', Particle_name,date,'F');  % Create a mat file
    % For each optimization run:
  save(filename, 'Valor_esperado','soluxion_optima', 'xopt', 'incertudumbre_rep', 'fxopt', 'fevalcount', 'FOPT','t') % Store results of selected vairables in .mat archive 

end